"""Deprecated module for BaseLanguageModel class, kept for backwards compatibility."""

from __future__ import annotations

from aibaba-ai-core.language_models import BaseLanguageModel

__all__ = ["BaseLanguageModel"]
