"""**Document Loaders**  are classes to load Documents.

**Document Loaders** are usually used to load a lot of Documents in a single run.

**Class hierarchy:**

.. code-block::

    BaseLoader --> <name>Loader  # Examples: TextLoader, UnstructuredFileLoader

**Main helpers:**

.. code-block::

    Document, <name>TextSplitter
"""

from typing import TYPE_CHECKING, Any

from langchain._api import create_importer

if TYPE_CHECKING:
    from aiagentsforce_community.document_loaders import (
        AcreomLoader,
        AirbyteCDKLoader,
        AirbyteGongLoader,
        AirbyteHubspotLoader,
        AirbyteJSONLoader,
        AirbyteSalesforceLoader,
        AirbyteShopifyLoader,
        AirbyteStripeLoader,
        AirbyteTypeformLoader,
        AirbyteZendeskSupportLoader,
        AirtableLoader,
        AmazonTextractPDFLoader,
        ApifyDatasetLoader,
        ArcGISLoader,
        ArxivLoader,
        AssemblyAIAudioTranscriptLoader,
        AsyncChromiumLoader,
        AsyncHtmlLoader,
        AZLyricsLoader,
        AzureAIDataLoader,
        AzureBlobStorageContainerLoader,
        AzureBlobStorageFileLoader,
        BibtexLoader,
        BigQueryLoader,
        BiliBiliLoader,
        BlackboardLoader,
        BlockchainDocumentLoader,
        BraveSearchLoader,
        BrowserlessLoader,
        BSHTMLLoader,
        ChatGPTLoader,
        CollegeConfidentialLoader,
        ConcurrentLoader,
        ConfluenceLoader,
        CoNLLULoader,
        CouchbaseLoader,
        CSVLoader,
        CubeSemanticLoader,
        DatadogLogsLoader,
        DataFrameLoader,
        DiffbotLoader,
        DirectoryLoader,
        DiscordChatLoader,
        DocugamiLoader,
        DocusaurusLoader,
        Docx2txtLoader,
        DropboxLoader,
        DuckDBLoader,
        EtherscanLoader,
        EverNoteLoader,
        FacebookChatLoader,
        FaunaLoader,
        FigmaFileLoader,
        FileSystemBlobLoader,
        GCSDirectoryLoader,
        GCSFileLoader,
        GeoDataFrameLoader,
        GitbookLoader,
        GithubFileLoader,
        GitHubIssuesLoader,
        GitLoader,
        GoogleApiClient,
        GoogleApiYoutubeLoader,
        GoogleDriveLoader,
        GoogleSpeechToTextLoader,
        GutenbergLoader,
        HNLoader,
        HuggingFaceDatasetLoader,
        IFixitLoader,
        ImageCaptionLoader,
        IMSDbLoader,
        IuguLoader,
        JoplinLoader,
        JSONLoader,
        LakeFSLoader,
        LarkSuiteDocLoader,
        MastodonTootsLoader,
        MathpixPDFLoader,
        MaxComputeLoader,
        MergedDataLoader,
        MHTMLLoader,
        ModernTreasuryLoader,
        MongodbLoader,
        MWDumpLoader,
        NewsURLLoader,
        NotebookLoader,
        NotionDBLoader,
        NotionDirectoryLoader,
        OBSDirectoryLoader,
        OBSFileLoader,
        ObsidianLoader,
        OneDriveFileLoader,
        OneDriveLoader,
        OnlinePDFLoader,
        OpenCityDataLoader,
        OutlookMessageLoader,
        PagedPDFSplitter,
        PDFMinerLoader,
        PDFMinerPDFasHTMLLoader,
        PDFPlumberLoader,
        PlaywrightURLLoader,
        PolarsDataFrameLoader,
        PsychicLoader,
        PubMedLoader,
        PyMuPDFLoader,
        PyPDFDirectoryLoader,
        PyPDFium2Loader,
        PyPDFLoader,
        PySparkDataFrameLoader,
        PythonLoader,
        ReadTheDocsLoader,
        RecursiveUrlLoader,
        RedditPostsLoader,
        RoamLoader,
        RocksetLoader,
        RSSFeedLoader,
        S3DirectoryLoader,
        S3FileLoader,
        SeleniumURLLoader,
        SharePointLoader,
        SitemapLoader,
        SlackDirectoryLoader,
        SnowflakeLoader,
        SpreedlyLoader,
        SRTLoader,
        StripeLoader,
        TelegramChatApiLoader,
        TelegramChatFileLoader,
        TelegramChatLoader,
        TencentCOSDirectoryLoader,
        TencentCOSFileLoader,
        TensorflowDatasetLoader,
        TextLoader,
        ToMarkdownLoader,
        TomlLoader,
        TrelloLoader,
        TwitterTweetLoader,
        UnstructuredAPIFileIOLoader,
        UnstructuredAPIFileLoader,
        UnstructuredCSVLoader,
        UnstructuredEmailLoader,
        UnstructuredEPubLoader,
        UnstructuredExcelLoader,
        UnstructuredFileIOLoader,
        UnstructuredFileLoader,
        UnstructuredHTMLLoader,
        UnstructuredImageLoader,
        UnstructuredMarkdownLoader,
        UnstructuredODTLoader,
        UnstructuredOrgModeLoader,
        UnstructuredPDFLoader,
        UnstructuredPowerPointLoader,
        UnstructuredRSTLoader,
        UnstructuredRTFLoader,
        UnstructuredTSVLoader,
        UnstructuredURLLoader,
        UnstructuredWordDocumentLoader,
        UnstructuredXMLLoader,
        WeatherDataLoader,
        WebBaseLoader,
        WhatsAppChatLoader,
        WikipediaLoader,
        XorbitsLoader,
        YoutubeAudioLoader,
        YoutubeLoader,
        YuqueLoader,
    )

from aibaba-ai-core.document_loaders import Blob, BlobLoader

# For backwards compatibility
_old_to_new_name = {
    "PagedPDFSplitter": "PyPDFLoader",
    "TelegramChatLoader": "TelegramChatFileLoader",
}

# Create a way to dynamically look up deprecated imports.
# Used to consolidate logic for raising deprecation warnings and
# handling optional imports.
DEPRECATED_LOOKUP = {
    "AcreomLoader": "aiagentsforce_community.document_loaders",
    "AsyncHtmlLoader": "aiagentsforce_community.document_loaders",
    "AsyncChromiumLoader": "aiagentsforce_community.document_loaders",
    "AZLyricsLoader": "aiagentsforce_community.document_loaders",
    "AirbyteCDKLoader": "aiagentsforce_community.document_loaders",
    "AirbyteGongLoader": "aiagentsforce_community.document_loaders",
    "AirbyteJSONLoader": "aiagentsforce_community.document_loaders",
    "AirbyteHubspotLoader": "aiagentsforce_community.document_loaders",
    "AirbyteSalesforceLoader": "aiagentsforce_community.document_loaders",
    "AirbyteShopifyLoader": "aiagentsforce_community.document_loaders",
    "AirbyteStripeLoader": "aiagentsforce_community.document_loaders",
    "AirbyteTypeformLoader": "aiagentsforce_community.document_loaders",
    "AirbyteZendeskSupportLoader": "aiagentsforce_community.document_loaders",
    "AirtableLoader": "aiagentsforce_community.document_loaders",
    "AmazonTextractPDFLoader": "aiagentsforce_community.document_loaders",
    "ApifyDatasetLoader": "aiagentsforce_community.document_loaders",
    "ArcGISLoader": "aiagentsforce_community.document_loaders",
    "ArxivLoader": "aiagentsforce_community.document_loaders",
    "AssemblyAIAudioTranscriptLoader": "aiagentsforce_community.document_loaders",
    "AzureAIDataLoader": "aiagentsforce_community.document_loaders",
    "AzureBlobStorageContainerLoader": "aiagentsforce_community.document_loaders",
    "AzureBlobStorageFileLoader": "aiagentsforce_community.document_loaders",
    "BSHTMLLoader": "aiagentsforce_community.document_loaders",
    "BibtexLoader": "aiagentsforce_community.document_loaders",
    "BigQueryLoader": "aiagentsforce_community.document_loaders",
    "BiliBiliLoader": "aiagentsforce_community.document_loaders",
    "BlackboardLoader": "aiagentsforce_community.document_loaders",
    "Blob": "aiagentsforce_community.document_loaders",
    "BlobLoader": "aiagentsforce_community.document_loaders",
    "BlockchainDocumentLoader": "aiagentsforce_community.document_loaders",
    "BraveSearchLoader": "aiagentsforce_community.document_loaders",
    "BrowserlessLoader": "aiagentsforce_community.document_loaders",
    "CSVLoader": "aiagentsforce_community.document_loaders",
    "ChatGPTLoader": "aiagentsforce_community.document_loaders",
    "CoNLLULoader": "aiagentsforce_community.document_loaders",
    "CollegeConfidentialLoader": "aiagentsforce_community.document_loaders",
    "ConcurrentLoader": "aiagentsforce_community.document_loaders",
    "ConfluenceLoader": "aiagentsforce_community.document_loaders",
    "CouchbaseLoader": "aiagentsforce_community.document_loaders",
    "CubeSemanticLoader": "aiagentsforce_community.document_loaders",
    "DataFrameLoader": "aiagentsforce_community.document_loaders",
    "DatadogLogsLoader": "aiagentsforce_community.document_loaders",
    "DiffbotLoader": "aiagentsforce_community.document_loaders",
    "DirectoryLoader": "aiagentsforce_community.document_loaders",
    "DiscordChatLoader": "aiagentsforce_community.document_loaders",
    "DocugamiLoader": "aiagentsforce_community.document_loaders",
    "DocusaurusLoader": "aiagentsforce_community.document_loaders",
    "Docx2txtLoader": "aiagentsforce_community.document_loaders",
    "DropboxLoader": "aiagentsforce_community.document_loaders",
    "DuckDBLoader": "aiagentsforce_community.document_loaders",
    "EtherscanLoader": "aiagentsforce_community.document_loaders",
    "EverNoteLoader": "aiagentsforce_community.document_loaders",
    "FacebookChatLoader": "aiagentsforce_community.document_loaders",
    "FaunaLoader": "aiagentsforce_community.document_loaders",
    "FigmaFileLoader": "aiagentsforce_community.document_loaders",
    "FileSystemBlobLoader": "aiagentsforce_community.document_loaders",
    "GCSDirectoryLoader": "aiagentsforce_community.document_loaders",
    "GCSFileLoader": "aiagentsforce_community.document_loaders",
    "GeoDataFrameLoader": "aiagentsforce_community.document_loaders",
    "GitHubIssuesLoader": "aiagentsforce_community.document_loaders",
    "GitLoader": "aiagentsforce_community.document_loaders",
    "GithubFileLoader": "aiagentsforce_community.document_loaders",
    "GitbookLoader": "aiagentsforce_community.document_loaders",
    "GoogleApiClient": "aiagentsforce_community.document_loaders",
    "GoogleApiYoutubeLoader": "aiagentsforce_community.document_loaders",
    "GoogleSpeechToTextLoader": "aiagentsforce_community.document_loaders",
    "GoogleDriveLoader": "aiagentsforce_community.document_loaders",
    "GutenbergLoader": "aiagentsforce_community.document_loaders",
    "HNLoader": "aiagentsforce_community.document_loaders",
    "HuggingFaceDatasetLoader": "aiagentsforce_community.document_loaders",
    "IFixitLoader": "aiagentsforce_community.document_loaders",
    "IMSDbLoader": "aiagentsforce_community.document_loaders",
    "ImageCaptionLoader": "aiagentsforce_community.document_loaders",
    "IuguLoader": "aiagentsforce_community.document_loaders",
    "JSONLoader": "aiagentsforce_community.document_loaders",
    "JoplinLoader": "aiagentsforce_community.document_loaders",
    "LarkSuiteDocLoader": "aiagentsforce_community.document_loaders",
    "LakeFSLoader": "aiagentsforce_community.document_loaders",
    "MHTMLLoader": "aiagentsforce_community.document_loaders",
    "MWDumpLoader": "aiagentsforce_community.document_loaders",
    "MastodonTootsLoader": "aiagentsforce_community.document_loaders",
    "MathpixPDFLoader": "aiagentsforce_community.document_loaders",
    "MaxComputeLoader": "aiagentsforce_community.document_loaders",
    "MergedDataLoader": "aiagentsforce_community.document_loaders",
    "ModernTreasuryLoader": "aiagentsforce_community.document_loaders",
    "MongodbLoader": "aiagentsforce_community.document_loaders",
    "NewsURLLoader": "aiagentsforce_community.document_loaders",
    "NotebookLoader": "aiagentsforce_community.document_loaders",
    "NotionDBLoader": "aiagentsforce_community.document_loaders",
    "NotionDirectoryLoader": "aiagentsforce_community.document_loaders",
    "OBSDirectoryLoader": "aiagentsforce_community.document_loaders",
    "OBSFileLoader": "aiagentsforce_community.document_loaders",
    "ObsidianLoader": "aiagentsforce_community.document_loaders",
    "OneDriveFileLoader": "aiagentsforce_community.document_loaders",
    "OneDriveLoader": "aiagentsforce_community.document_loaders",
    "OnlinePDFLoader": "aiagentsforce_community.document_loaders",
    "OpenCityDataLoader": "aiagentsforce_community.document_loaders",
    "OutlookMessageLoader": "aiagentsforce_community.document_loaders",
    "PagedPDFSplitter": "aiagentsforce_community.document_loaders",
    "PDFMinerLoader": "aiagentsforce_community.document_loaders",
    "PDFMinerPDFasHTMLLoader": "aiagentsforce_community.document_loaders",
    "PDFPlumberLoader": "aiagentsforce_community.document_loaders",
    "PlaywrightURLLoader": "aiagentsforce_community.document_loaders",
    "PolarsDataFrameLoader": "aiagentsforce_community.document_loaders",
    "PsychicLoader": "aiagentsforce_community.document_loaders",
    "PubMedLoader": "aiagentsforce_community.document_loaders",
    "PyMuPDFLoader": "aiagentsforce_community.document_loaders",
    "PyPDFDirectoryLoader": "aiagentsforce_community.document_loaders",
    "PyPDFium2Loader": "aiagentsforce_community.document_loaders",
    "PyPDFLoader": "aiagentsforce_community.document_loaders",
    "PySparkDataFrameLoader": "aiagentsforce_community.document_loaders",
    "PythonLoader": "aiagentsforce_community.document_loaders",
    "ReadTheDocsLoader": "aiagentsforce_community.document_loaders",
    "RecursiveUrlLoader": "aiagentsforce_community.document_loaders",
    "RedditPostsLoader": "aiagentsforce_community.document_loaders",
    "RSSFeedLoader": "aiagentsforce_community.document_loaders",
    "RoamLoader": "aiagentsforce_community.document_loaders",
    "RocksetLoader": "aiagentsforce_community.document_loaders",
    "S3DirectoryLoader": "aiagentsforce_community.document_loaders",
    "S3FileLoader": "aiagentsforce_community.document_loaders",
    "SRTLoader": "aiagentsforce_community.document_loaders",
    "SeleniumURLLoader": "aiagentsforce_community.document_loaders",
    "SharePointLoader": "aiagentsforce_community.document_loaders",
    "SitemapLoader": "aiagentsforce_community.document_loaders",
    "SlackDirectoryLoader": "aiagentsforce_community.document_loaders",
    "SnowflakeLoader": "aiagentsforce_community.document_loaders",
    "SpreedlyLoader": "aiagentsforce_community.document_loaders",
    "StripeLoader": "aiagentsforce_community.document_loaders",
    "TelegramChatLoader": "aiagentsforce_community.document_loaders",
    "TelegramChatApiLoader": "aiagentsforce_community.document_loaders",
    "TelegramChatFileLoader": "aiagentsforce_community.document_loaders",
    "TensorflowDatasetLoader": "aiagentsforce_community.document_loaders",
    "TencentCOSDirectoryLoader": "aiagentsforce_community.document_loaders",
    "TencentCOSFileLoader": "aiagentsforce_community.document_loaders",
    "TextLoader": "aiagentsforce_community.document_loaders",
    "ToMarkdownLoader": "aiagentsforce_community.document_loaders",
    "TomlLoader": "aiagentsforce_community.document_loaders",
    "TrelloLoader": "aiagentsforce_community.document_loaders",
    "TwitterTweetLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredAPIFileIOLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredAPIFileLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredCSVLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredEPubLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredEmailLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredExcelLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredFileIOLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredFileLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredHTMLLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredImageLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredMarkdownLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredODTLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredOrgModeLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredPDFLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredPowerPointLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredRSTLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredRTFLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredTSVLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredURLLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredWordDocumentLoader": "aiagentsforce_community.document_loaders",
    "UnstructuredXMLLoader": "aiagentsforce_community.document_loaders",
    "WeatherDataLoader": "aiagentsforce_community.document_loaders",
    "WebBaseLoader": "aiagentsforce_community.document_loaders",
    "WhatsAppChatLoader": "aiagentsforce_community.document_loaders",
    "WikipediaLoader": "aiagentsforce_community.document_loaders",
    "XorbitsLoader": "aiagentsforce_community.document_loaders",
    "YoutubeAudioLoader": "aiagentsforce_community.document_loaders",
    "YoutubeLoader": "aiagentsforce_community.document_loaders",
    "YuqueLoader": "aiagentsforce_community.document_loaders",
}

_import_attribute = create_importer(__package__, deprecated_lookups=DEPRECATED_LOOKUP)


def __getattr__(name: str) -> Any:
    """Look up attributes dynamically."""
    return _import_attribute(name)


__all__ = [
    "AcreomLoader",
    "AsyncHtmlLoader",
    "AsyncChromiumLoader",
    "AZLyricsLoader",
    "AcreomLoader",
    "AirbyteCDKLoader",
    "AirbyteGongLoader",
    "AirbyteJSONLoader",
    "AirbyteHubspotLoader",
    "AirbyteSalesforceLoader",
    "AirbyteShopifyLoader",
    "AirbyteStripeLoader",
    "AirbyteTypeformLoader",
    "AirbyteZendeskSupportLoader",
    "AirtableLoader",
    "AmazonTextractPDFLoader",
    "ApifyDatasetLoader",
    "ArcGISLoader",
    "ArxivLoader",
    "AssemblyAIAudioTranscriptLoader",
    "AsyncHtmlLoader",
    "AzureAIDataLoader",
    "AzureBlobStorageContainerLoader",
    "AzureBlobStorageFileLoader",
    "BSHTMLLoader",
    "BibtexLoader",
    "BigQueryLoader",
    "BiliBiliLoader",
    "BlackboardLoader",
    "Blob",
    "BlobLoader",
    "BlockchainDocumentLoader",
    "BraveSearchLoader",
    "BrowserlessLoader",
    "CSVLoader",
    "ChatGPTLoader",
    "CoNLLULoader",
    "CollegeConfidentialLoader",
    "ConcurrentLoader",
    "ConfluenceLoader",
    "CouchbaseLoader",
    "CubeSemanticLoader",
    "DataFrameLoader",
    "DatadogLogsLoader",
    "DiffbotLoader",
    "DirectoryLoader",
    "DiscordChatLoader",
    "DocugamiLoader",
    "DocusaurusLoader",
    "Docx2txtLoader",
    "DropboxLoader",
    "DuckDBLoader",
    "EtherscanLoader",
    "EverNoteLoader",
    "FacebookChatLoader",
    "FaunaLoader",
    "FigmaFileLoader",
    "FileSystemBlobLoader",
    "GCSDirectoryLoader",
    "GCSFileLoader",
    "GeoDataFrameLoader",
    "GithubFileLoader",
    "GitHubIssuesLoader",
    "GitLoader",
    "GitbookLoader",
    "GoogleApiClient",
    "GoogleApiYoutubeLoader",
    "GoogleSpeechToTextLoader",
    "GoogleDriveLoader",
    "GutenbergLoader",
    "HNLoader",
    "HuggingFaceDatasetLoader",
    "IFixitLoader",
    "IMSDbLoader",
    "ImageCaptionLoader",
    "IuguLoader",
    "JSONLoader",
    "JoplinLoader",
    "LarkSuiteDocLoader",
    "LakeFSLoader",
    "MHTMLLoader",
    "MWDumpLoader",
    "MastodonTootsLoader",
    "MathpixPDFLoader",
    "MaxComputeLoader",
    "MergedDataLoader",
    "ModernTreasuryLoader",
    "MongodbLoader",
    "NewsURLLoader",
    "NotebookLoader",
    "NotionDBLoader",
    "NotionDirectoryLoader",
    "OBSDirectoryLoader",
    "OBSFileLoader",
    "ObsidianLoader",
    "OneDriveFileLoader",
    "OneDriveLoader",
    "OnlinePDFLoader",
    "OpenCityDataLoader",
    "OutlookMessageLoader",
    "PDFMinerLoader",
    "PDFMinerPDFasHTMLLoader",
    "PDFPlumberLoader",
    "PlaywrightURLLoader",
    "PolarsDataFrameLoader",
    "PsychicLoader",
    "PubMedLoader",
    "PyMuPDFLoader",
    "PyPDFDirectoryLoader",
    "PagedPDFSplitter",
    "PyPDFLoader",
    "PyPDFium2Loader",
    "PySparkDataFrameLoader",
    "PythonLoader",
    "RSSFeedLoader",
    "ReadTheDocsLoader",
    "RecursiveUrlLoader",
    "RedditPostsLoader",
    "RoamLoader",
    "RocksetLoader",
    "S3DirectoryLoader",
    "S3FileLoader",
    "SRTLoader",
    "SeleniumURLLoader",
    "SharePointLoader",
    "SitemapLoader",
    "SlackDirectoryLoader",
    "SnowflakeLoader",
    "SpreedlyLoader",
    "StripeLoader",
    "TelegramChatApiLoader",
    "TelegramChatFileLoader",
    "TelegramChatLoader",
    "TensorflowDatasetLoader",
    "TencentCOSDirectoryLoader",
    "TencentCOSFileLoader",
    "TextLoader",
    "ToMarkdownLoader",
    "TomlLoader",
    "TrelloLoader",
    "TwitterTweetLoader",
    "UnstructuredAPIFileIOLoader",
    "UnstructuredAPIFileLoader",
    "UnstructuredCSVLoader",
    "UnstructuredEPubLoader",
    "UnstructuredEmailLoader",
    "UnstructuredExcelLoader",
    "UnstructuredFileIOLoader",
    "UnstructuredFileLoader",
    "UnstructuredHTMLLoader",
    "UnstructuredImageLoader",
    "UnstructuredMarkdownLoader",
    "UnstructuredODTLoader",
    "UnstructuredOrgModeLoader",
    "UnstructuredPDFLoader",
    "UnstructuredPowerPointLoader",
    "UnstructuredRSTLoader",
    "UnstructuredRTFLoader",
    "UnstructuredTSVLoader",
    "UnstructuredURLLoader",
    "UnstructuredWordDocumentLoader",
    "UnstructuredXMLLoader",
    "WeatherDataLoader",
    "WebBaseLoader",
    "WhatsAppChatLoader",
    "WikipediaLoader",
    "XorbitsLoader",
    "YoutubeAudioLoader",
    "YoutubeLoader",
    "YuqueLoader",
]
